!{\src2tex{textfont=tt}}
!!****f* ABINIT/mka2f
!!
!! NAME
!! mka2f
!!
!! FUNCTION
!!  calculate the FS averaged alpha^2F function
!!
!! COPYRIGHT
!! Copyright (C) 2004 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  elph_ds
!!    elph_ds%gkk2 = gkk2 matrix elements on full FS grid for each phonon mode
!!    elph_ds%nbranch = number of phonon branches = 3*natom
!!    elph_ds%nFSband = number of bands included in the FS integration
!!    elph_ds%nFSkpt = number of kpts included in the FS integration
!!    elph_ds%phfrq = phonon frequencies
!!  FSintweight = integration weights on the FS
!!  FSirredtofull = indices of irreducible FS kpoints in full FSkpt array
!!  FSirredwtk = weights of irreducible kpoints
!!  FSkpt = coordinates of all FS kpoints
!!  gprimd = reciprocal lattice vectors (dimensionful)
!!  mustar = coulomb pseudopotential parameter
!!  n0 = DOS at the Fermi level calculated from the FSkpt integration weights
!!  natom = number of atoms
!!  nomega = number of frequencies at which to calculate a2f
!!  nrpt = number of real-space points for FT interpolation
!!  phon_ds = datastructure with interatomic force constants to interpolate
!!     phonons
!!  rpt = coordinates of real-space points for FT interpolation
!!  wghatm = weights for real-space points for FT interpolation
!!
!! OUTPUT
!!  elph_ds
!!    elph_ds%a2f = alpha^2F function on full FS grid : omega,iFSkpt,iqpt
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!      chpev,ftgam,inpphon,simpson_int,zhpev
!!
!! NOTES
!!   copied from ftiaf9.f
!!
!! SOURCE

 subroutine mka2f(elph_ds,FSintweight,FSirredtofull,FSirredwtk,FSkpt,gprimd,mustar,&
&      n0,natom,nomega,nrpt,phon_ds,rpt,wghatm)

 use defs_basis
 use defs_datatypes
 use defs_elphon
 
 implicit none
 
!Arguments ------------------------------------
 integer,intent(in) :: natom,nomega,nrpt
 real(dp),intent(in) :: mustar,n0
 type(elph_type),intent(inout) :: elph_ds
 type(phon_type),intent(in) :: phon_ds
 integer,intent(in) :: FSirredtofull(elph_ds%nFSkptirred)
 real(dp),intent(in) :: FSintweight(elph_ds%nFSband,elph_ds%nFSkpt)
 real(dp),intent(in) :: FSirredwtk(elph_ds%nFSkptirred),FSkpt(3,elph_ds%nFSkpt)
 real(dp),intent(in) :: gprimd(3,3),rpt(3,nrpt),wghatm(natom,natom,nrpt)
 
!Local variables -------------------------
 integer :: i1,i2,iFSqpt,ib1,ib2,ibranch,ieqFSkpt1,ier,ii,iomega,iost,ip,iqpt
 integer :: irpt,mu,nu,unit_a2f
 real(dp) :: a2fprefactor,domega,gaussfactor,gaussprefactor,gaussval,lambda_iso
 real(dp) :: omega,omega_max,omega_smear,omegalog,tc_macmill,xx
 character(len=fnlen) :: fname
 real(dp) :: a2f_integ(nomega),displ(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: eigval(elph_ds%nbranch),eigvec(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: gam_now(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: pheigval(elph_ds%nbranch)
 real(dp) :: pheigvec(2*elph_ds%nbranch*elph_ds%nbranch),phfrq(elph_ds%nbranch)
 real(dp) :: tmp_gkk2(elph_ds%nbranch,elph_ds%nFSkpt),tmpa2f(nomega)
 real(dp),allocatable :: a2f_1mom(:),a2f_1mom_int(:),a2flogmom(:)
 real(dp),allocatable :: a2flogmom_int(:),matrx(:,:),zhpev1(:,:),zhpev2(:)
 
! *********************************************************************
! calculate a2f for frequencies between 0 and omega_max

#ifdef __VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif
 
  write (*,*) 'mka2f : enter '

  !allocate (elph_ds%a2f(nomega,elph_ds%nFSkpt,elph_ds%nFSkpt))

  ! smearing width for deltas in the expression for alpha^2 F
  omega_smear = 0.00001_dp

  ! maximum value of frequency (a grid has to be chosen for the representation of alpha^2 F)
  omega_max = maxval(elph_ds%phfrq) + 20.0_dp*omega_smear
  omega_max = 0.002

  gaussprefactor = sqrt(piinv) / omega_smear
  gaussfactor = one / omega_smear
  domega = omega_max/(nomega-one)

!DEBUG
!write (*,*) ' mka2f : FSirredwtk'
!write (*,'(6(E16.6))') FSirredwtk
!write (*,*) ' mka2f :sum(FSintweight) = ',sum(FSintweight)
!write (*,*) ' mka2f : omega_max,omega_smear,nomega ', &
!       & omega_max,omega_smear,nomega
!  write (*,'(6(E16.6))') gaussprefactor,gaussfactor,domega
!ENDDEBUG

  allocate(matrx(2,(3*natom*(3*natom+1))/2))
  allocate(zhpev1(2,2*3*natom-1),zhpev2(3*3*natom-2))

  a2f_integ(:) = zero
  !elph_ds%a2f(:,:,:) = zero

  ! loop over qpoint
  do iFSqpt=1,elph_ds%nFSkpt
!DEBUG
!write (*,*) 'mka2f : iFSqpt = ', iFSqpt, ' / ', elph_ds%nFSkpt
!ENDDEBUG

     ! This reduced version of ftgkk supposes the kpoints have been integrated
     !   in integrate_gamma. Do FT from real-space gamma grid to 1 qpt.
     call ftgam(wghatm,gam_now,elph_ds%gamma_rpt,gprimd,natom,1,nrpt,0,rpt,&
      &  FSkpt(:,iFSqpt))

     ! Diagonalize gamma matrix at qpoint (complex matrix). Copied from phfrq33
     ier=0
     ii=1
     do i2=1,3*natom
        do i1=1,i2
           matrx(1,ii)=gam_now(1,i1,i2)
           matrx(2,ii)=gam_now(2,i1,i2)
           ii=ii+1
        end do
     end do
#if defined T3E
     call CHPEV ('N','U',3*natom,matrx,eigval,eigvec,3*natom,zhpev1,&
          &    zhpev2,ier)
#else
     call ZHPEV ('N','U',3*natom,matrx,eigval,eigvec,3*natom,zhpev1,&
          &    zhpev2,ier)
#endif

     call inpphon(displ,pheigval,pheigvec,phfrq,phon_ds,FSkpt(:,iFSqpt))

     do ibranch=1,elph_ds%nbranch

        if (phfrq(ibranch) < 0.00005) cycle
        a2fprefactor = eigval(ibranch)/(two_pi*phfrq(ibranch)*n0)

        omega = zero
        tmpa2f(:) = zero
        do iomega=1,nomega
           xx = (omega-phfrq(ibranch))*gaussfactor
           gaussval = gaussprefactor*exp(-xx*xx)

           tmpa2f(iomega) = tmpa2f(iomega) + gaussval*a2fprefactor

           omega = omega+domega
        end do

        a2f_integ(:) = a2f_integ(:) + tmpa2f(:)

     end do
     ! end ibranch do

  end do
  ! end iFSqpt do

  a2f_integ(:) = a2f_integ(:) / elph_ds%nFSkpt

  deallocate(matrx,zhpev1,zhpev2)

  !write (*,'(6(E16.6,2x))') a2f_integ(:)

  ! output the a2f_integ
  unit_a2f = 108
  fname = trim(elph_ds%elph_base_name) // "_A2F"
  open (UNIT=unit_a2f,FILE=fname,STATUS='REPLACE')

  write (*,*) ' a2f function integrated over the FS'
  write (unit_a2f,'(a)') '# a2f function integrated over the FS. omega in a.u.'
  omega = zero
  do iomega=1,nomega
     write (*,*) omega, a2f_integ(iomega)
     write (unit_a2f,'(2E16.6)') omega, a2f_integ(iomega)
     omega=omega+domega
  end do

  close (UNIT=unit_a2f)

  ! Do isotropic calculation of lambda and output lambda, Tc(MacMillan)

  allocate (a2f_1mom(nomega),a2f_1mom_int(nomega))
  omega = domega
  a2f_1mom(1) = zero
  do iomega=2,nomega
     a2f_1mom(iomega) = two*a2f_integ(iomega)/omega
     omega=omega+domega
  end do
  call simpson_int(nomega,domega,a2f_1mom,a2f_1mom_int)
  lambda_iso = a2f_1mom_int(nomega)

  deallocate (a2f_1mom,a2f_1mom_int)

  write (*,*) 'mka2f: isotropic lambda = ', lambda_iso


  ! Get log moment of alpha^2F
  allocate (a2flogmom(nomega),a2flogmom_int(nomega))
  omega = domega
  a2flogmom(1) = zero
  do iomega=2,nomega
     a2flogmom(iomega) = (two/lambda_iso)*a2f_integ(iomega)*log(omega)/omega
     omega=omega+domega
  end do
  call simpson_int(nomega,domega,a2flogmom,a2flogmom_int)
  omegalog = exp(a2flogmom_int(nomega))

  deallocate (a2flogmom,a2flogmom_int)

  write (*,*) 'mka2f: omegalog  = ', omegalog, ' (Ha) ', omegalog/kb_HaK, ' (Kelvin) '
  write (*,*) 'mka2f: input mustar = ', mustar

  tc_macmill = omegalog/1.2_dp * exp((         -1.04_dp*(one+lambda_iso)        ) /&
       &                             (lambda_iso-mustar*(one+0.62_dp*lambda_iso)))

  write (*,*) 'mka2f: MacMillan Tc = ', tc_macmill, ' (Ha) ', tc_macmill/kb_HaK, ' (Kelvin) '

  write (*,*) ' mka2f : end '

 end subroutine mka2f
!!***
