!{\src2tex{textfont=tt}}
!!****f* ABINIT/mkelph_linwid
!!
!! NAME
!! mkelph_linwid
!!
!! FUNCTION
!!  Calculate the phonon linewidths on a trajectory in q space
!!
!! COPYRIGHT
!! Copyright (C) 2004 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  elph_ds = datastructure with phonon matrix elements
!!  FSintweight = integration weights on the FS
!!  FSfulltoirred = mapping of full FS kpts to irreducible ones
!!  FSirredtofull = indices of irreducible kpoints in the full ones
!!  FStoqpt = mapping of kpoint pairs to qpoints
!!  gkk2 = gkk2 matrix elements on full FS grid for each phonon mode
!!  gmet = reciprocal space metric
!!  gprimd = reciprocal lattice vectors
!!  n0 = DOS at the Fermi level calculated from the FSkpt integration weights
!!  natom = number of atoms
!!  npoint_in = number of points requested along trajectory
!!  nrpt = number of real space points for FT interpolation
!!  nsegment_in = number of segments in reciprocal space trajectory
!!  nsym = number of symops
!!  phon_ds = datastructure with interatomic force constants
!!  qpath_vertices_in = vertices of reciprocal space trajectory
!!  qpttoqpt = mapping of qpoints under symops
!!  rpt = coordinates of real space points for FT interpolation
!!  spqpt = coordinates of qpoints
!!  wghatm = weights of pairs of atoms for FT interpolation
!!  rprimd = real-space lattice vectors (dimensionful)
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!      canon9,chpev,ftgam,inpphon,zhpev
!!
!! NOTES
!!
!! SOURCE

 subroutine mkelph_linwid(elph_ds,FSintweight,FSfulltoirred,FSirredtofull,FStoqpt,&
& gmet,gprimd,n0,natom,npoint_in,nrpt,nsegment_in,nsym,phon_ds,&
& qpath_vertices_in,qpttoqpt,rpt,spqpt,wghatm,rprimd)

 use defs_basis
 use defs_datatypes
 use defs_elphon
 
 implicit none
 
!Arguments ------------------------------------
  ! needed for phonon interpolation
 integer,intent(in) :: natom,nrpt,nsegment_in,nsym
 real(dp),intent(in) :: n0
 type(elph_type),intent(in) :: elph_ds
 type(phon_type),intent(in) :: phon_ds
 integer,intent(in) :: FSfulltoirred(3,elph_ds%nFSkpt)
 integer,intent(in) :: FSirredtofull(elph_ds%nFSkptirred)
 integer,intent(in) :: FStoqpt(elph_ds%nFSkpt,elph_ds%nFSkpt)
 integer,intent(in) :: npoint_in(nsegment_in),qpttoqpt(2,nsym,elph_ds%nqpt)
 real(dp),intent(in) :: FSintweight(elph_ds%nFSband,elph_ds%nFSkpt),gmet(3,3)
 real(dp),intent(in) :: gprimd(3,3),qpath_vertices_in(3,nsegment_in+1)
 real(dp),intent(in) :: rprimd(3,3),rpt(3,nrpt),spqpt(3,elph_ds%nqpt)
 real(dp),intent(in) :: wghatm(natom,natom,nrpt)
 
!Local variables-------------------------------
  ! for diagonalization of gammma matrix
  ! output variables for gtdyn9+phfrq3
 integer :: eivec=1,i1,i2,iFSkpt1,iFSkpt2,iFSkpt3,iatom,ib1,ib2,ibranch,idir
 integer :: ieqFSkpt1,ier,ii,indx,iost,ip,ipoint,iqpt,iqptfull,irpt,iseg
 integer :: jbranch,k1,kbranch,kdir,mu,nsegment,nu,qtor,unit_lwd
 real(dp) :: dist,gaussfactor,gaussprefactor,gaussval,phnow,qphnrm=one,res
 real(dp) :: total_weight,weight
 character(len=fnlen) :: fname
 integer :: npoint(nsegment_in+1)
 real(dp) :: displ(2,3*natom,3*natom)
 real(dp) :: displ_red(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: eigval(elph_ds%nbranch),eigvec(2*elph_ds%nbranch*elph_ds%nbranch)
 real(dp) :: elph_linwid_integ(elph_ds%nbranch,elph_ds%nqpt)
 real(dp) :: gam_now(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: gam_now2(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: gam_rpt(2,elph_ds%nbranch,elph_ds%nbranch),pheigval(3*natom)
 real(dp) :: pheigvec(2*3*natom*3*natom),phfrq_tmp(3*natom)
 real(dp) :: qpath_vertices(3,nsegment_in+2),qpt(3),redkpt(3)
 real(dp) :: tmp_gkk2(elph_ds%nbranch,elph_ds%nFSband,elph_ds%nFSband,elph_ds%nFSkpt)
 real(dp) :: tmpelph_linwid(elph_ds%nbranch),tmpqpt(3)
 real(dp),allocatable :: matrx(:,:),zhpev1(:,:),zhpev2(:)
 
! *********************************************************************
! calculate elph_linwid for qpoints along path defined by qpath_vertices

#ifdef __VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif

  write (*,*) 'mkelph_linwid : enter '

  ! add extra segment for last point: nvertices = nsegments + 1
  qpath_vertices(:,1:nsegment_in+1) = qpath_vertices_in(:,:)
  qpath_vertices(:,nsegment_in+2) = qpath_vertices_in(:,nsegment_in+1)
  npoint(1:nsegment_in) = npoint_in(:)
  npoint(nsegment_in+1) = 1
  nsegment=nsegment_in + 1

  unit_lwd=108
  fname=trim(elph_ds%elph_base_name) // "_LWD"
  open (UNIT=unit_lwd,FILE=fname,STATUS='REPLACE')
  indx=1

  ! real space to q space
  qtor=0

  write (*,*) ' mkelph_linwid : shape(elph_ds%gamma_qpt) = ',&
       &  shape(elph_ds%gamma_qpt)

!DEBUG
!write (100,*) '# gamma_qpt on qpts actually calculated = '
!do iqpt=1,elph_ds%nqpt
!   write (100,*) iqpt, spqpt(:,iqpt)
!   write (100,'(9(2E14.6,x))') elph_ds%gamma_qpt(:,:,:,iqpt)
!end do
!write (99,*) '# gamma_rpt on rpts actually calculated = '
!do irpt=1,nrpt
!   write (99,*) irpt, rpt(:,irpt)
!   write (99,'(9(2E14.6,x))') elph_ds%gamma_rpt(:,:,:,irpt)
!end do
!ENDDEBUG

! Back FT real space integrated values to recip space
!DEBUG
!write (95,*) '# difference btw gamma_qpt and FTed from real space = '
!qtor = 0
!do iqpt=1,elph_ds%nqpt
! call ftgam(wghatm,gam_now,elph_ds%gamma_rpt,gprimd,natom,1,nrpt,qtor,rpt,spqpt(:,iqpt))
! write (95,*) gam_now(:,:,:)-elph_ds%gamma_qpt(:,:,:,iqpt)
!end do
!write (94,*) '# difference btw gamma_rpt and back FTed from recip space = '
!qtor = 1
!do irpt=1,nrpt
! call ftgam(wghatm,elph_ds%gamma_qpt,gam_rpt,gprimd,natom,elph_ds%nqpt,1,qtor,rpt(:,irpt),spqpt)
! write (94,*) gam_rpt(:,:,:)-elph_ds%gamma_rpt(:,:,:,iqpt)
! elph_ds%gamma_rpt(:,:,:,irpt) = gam_rpt(:,:,:)
!end do
!write (95,*) '# explicit ftgam is done '
!qtor = 0
!do iqpt=1,elph_ds%nqpt
! call ftgam(wghatm,gam_now,elph_ds%gamma_rpt,gprimd,natom,1,nrpt,qtor,rpt,spqpt(:,iqpt))
! write (95,*) gam_now(:,:,:)-elph_ds%gamma_qpt(:,:,:,iqpt)
!end do
!ENDDEBUG


  !Interpolation along specified path in q space

  ! get reference distance
  tmpqpt(:) = half
  dist =     gmet(1,1)*tmpqpt(1)*tmpqpt(1) +&
       &     gmet(2,2)*tmpqpt(2)*tmpqpt(2) +&
       &     gmet(3,3)*tmpqpt(3)*tmpqpt(3) +&
       & two*gmet(1,2)*tmpqpt(1)*tmpqpt(2) +&
       & two*gmet(1,3)*tmpqpt(1)*tmpqpt(3) +&
       & two*gmet(2,3)*tmpqpt(2)*tmpqpt(3)
  gaussfactor=0.01_dp / dist / dist

!DEBUG
!write (100,*) '#-------------------------------------------------------------'
!write (100,*) '#NOW INTERPOLATED VALUES OF gamma MATRICES'
!write (100,*) '#-------------------------------------------------------------'
!
!write (96,*) '#displ_red vector at each qpoint :'
!write (97,*) '#displ vector at each qpoint :'
!write (98,*) '#bare gamma :'
!ENDDEBUG

  write (*,*) ' mkelph_linwid : elph_ds%doscalprod = ', elph_ds%doscalprod

  qtor = 0
  do iseg=1,nsegment
     do ipoint=0,npoint(iseg)-1

        !  Get qpoint along the path from qpath_vertices(:,iseg)
        !     to the point just before qpath_vertices(:,iseg+1)
!DEBUG
!write (*,*) 'mkelph_linwid : i and qpts ', iseg, ipoint, npoint(iseg)
!ENDDEBUG

        qpt(:) = qpath_vertices(:,iseg) + dble(ipoint)/dble(npoint(iseg))*&
             &  (qpath_vertices(:,iseg+1)-qpath_vertices(:,iseg))

!DEBUG
!write (*,'(3(3E16.6,x))') qpath_vertices(:,iseg), qpt(:), qpath_vertices(:,iseg+1)
!ENDDEBUG

        call canon9(qpt(1),redkpt(1),res)
        call canon9(qpt(2),redkpt(2),res)
        call canon9(qpt(3),redkpt(3),res)
        qpt(:) = redkpt(:)


        ! This reduced version of ftgkk supposes the kpoints have been integrated
        !   in integrate_gamma. Do FT from real-space gamma grid to 1 qpt.
        call ftgam(wghatm,gam_now,elph_ds%gamma_rpt,gprimd,natom,1,nrpt,qtor,rpt,qpt)

        !  if the matrices do not contain the scalar product with the displ vectors yet:
        if (elph_ds%doscalprod == 0) then

          call inpphon(displ,pheigval,pheigvec,phfrq_tmp,phon_ds,qpt)

!DEBUG
!write (*,*) 'phonon freqs ', phfrq_tmp
!ENDDEBUG

          displ_red(:,:,:) = zero
          do jbranch=1,elph_ds%nbranch
             do iatom=1,natom
                do idir=1,3
                   ibranch=idir+3*(iatom-1)
                   do kdir=1,3
                     k1 = kdir+3*(iatom-1)
                     !   WARNING: could be non-transpose of rprimd matrix : to be checked.
                     displ_red(1,ibranch,jbranch) = &
                        &   displ_red(1,ibranch,jbranch) + &
                        &   rprimd(kdir,idir)*displ(1,k1,jbranch)
                       !&   rprimd(kdir,idir)*displ(2*(k1 + 3*natom*(jbranch-1)) - 1)
                     displ_red(2,ibranch,jbranch) = &
                        &   displ_red(2,ibranch,jbranch) + &
                        &   rprimd(kdir,idir)*displ(2,k1,jbranch)
                       !&   rprimd(kdir,idir)*displ(2*(k1 + 3*natom*(jbranch-1))    )
                   end do
                end do
             end do
          end do

!DEBUG
!write (97,'(9(2E14.6,x))')  displ(:,:,:)
!write (96,'(9(2E14.6,x))')  displ_red(:,:,:)
!write (98,'(9(2E14.6,x))')  gam_now(:,:,:)
!ENDDEBUG

          gam_now2(:,:,:) = zero

          do jbranch=1,elph_ds%nbranch
             do ibranch=1,elph_ds%nbranch
                do kbranch=1,elph_ds%nbranch
                   ! gam = displ gam_red displ^{*T}
                   gam_now2(1,jbranch,jbranch) = gam_now2(1,jbranch,jbranch) + &
                        &  displ_red(1,ibranch,jbranch)*gam_now(1,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                        & +displ_red(1,ibranch,jbranch)*gam_now(2,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                        & +displ_red(2,ibranch,jbranch)*gam_now(1,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                        & -displ_red(2,ibranch,jbranch)*gam_now(2,ibranch,kbranch)*displ_red(1,kbranch,jbranch)
                   gam_now2(2,jbranch,jbranch) = gam_now2(2,jbranch,jbranch) + &
                        &  displ_red(2,ibranch,jbranch)*gam_now(2,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                        & +displ_red(2,ibranch,jbranch)*gam_now(1,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                        & +displ_red(1,ibranch,jbranch)*gam_now(2,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                        & -displ_red(1,ibranch,jbranch)*gam_now(1,ibranch,kbranch)*displ_red(2,kbranch,jbranch)
                end do
             end do
          end do

          gam_now(:,:,:) = gam_now2(:,:,:)
        end if
        ! end elph_ds%doscalprod if

!DEBUG
!write (100,'(9(2E14.6,x))') gam_now(:,:,:)
!ENDDEBUG

        ! Diagonalize gamma matrix at qpoint (complex matrix). Copied from phfrq33
        !! Useless with new formulation and exact displ vector at qpoint :
        !!    gam_now is already diagonal if doscalprod==0
        ier=0
        ii=1
        allocate(matrx(2,(3*natom*(3*natom+1))/2))
        do i2=1,3*natom
           do i1=1,i2
              matrx(1,ii)=gam_now(1,i1,i2)
              matrx(2,ii)=gam_now(2,i1,i2)
              ii=ii+1
           end do
        end do
        allocate(zhpev1(2,2*3*natom-1),zhpev2(3*3*natom-2))
#if defined T3E
        call CHPEV ('V','U',3*natom,matrx,eigval,eigvec,3*natom,zhpev1,&
             &    zhpev2,ier)
#else
        call ZHPEV ('V','U',3*natom,matrx,eigval,eigvec,3*natom,zhpev1,&
             &    zhpev2,ier)
#endif
        deallocate(matrx,zhpev1,zhpev2)

        !write (*,'(a,i5,i5,3E16.5,2x,6E16.6)',ADVANCE='NO') '---', iseg, ipoint, qpt, eigval
        write (unit_lwd,'(i5)', ADVANCE='NO') indx
        indx = indx+1
        write (unit_lwd,'(3E16.5)',ADVANCE='NO') eigval
        write (unit_lwd,*)

     end do
!  end ipoint do
  end do
!  end iseg do

  close (UNIT=unit_lwd)

  write (*,*) ' elph_linwid : end '


 end subroutine mkelph_linwid
!!***
