!{\src2tex{textfont=tt}}
!!****f* ABINIT/normsq_gkq
!!
!! NAME
!! normsq_gkq
!!
!! FUNCTION
!! This routine takes the gkq matrix elements for a given qpoint,
!!   does the scalar product with the phonon displacement vector,
!!   squares the gkq matrix elements
!!   multiplies by the appropriate weights and
!!   puts them in a uniform (atom,icart) basis
!!
!! COPYRIGHT
!! Copyright (C) 2004 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!   displ_red = phonon mode displacement vectors, post-multiplied by gprim matrix
!!     (ie. turned to reduced coordinates)
!!   eigvec =
!!   elph_ds = datastructure with gkk matrix elements
!!   FSintweight = FS integration weights for each band and kpt
!!   FSkpt = coordinates of kpoints near to FS
!!   h1_mat_el = matrix elements $<psi_{k+q,m} | H^{1} | psi_{k,n}>$
!!   iqptfull = index of present qpoint
!!   nband = number of bands
!!   phfrq_tmp = phonon frequencies
!!   spqpt = array of qpoint coordinates
!!   wf = gkk matrix element weight with $1/\sqrt{2 M \omega}$
!!
!! OUTPUT
!!   elph_ds%gkq filled
!!
!! NOTES
!!
!! PARENTS
!!      read_gkk
!!
!! CHILDREN
!!      canon9,chpev,zhpev
!!
!! SOURCE

 subroutine normsq_gkq(displ_red,eigvec,elph_ds,FSintweight,FSkpt,h1_mat_el,iqptfull,nband,phfrq_tmp,spqpt,wf)

 use defs_basis
 use defs_datatypes
 use defs_elphon
 
 implicit none
 
!Arguments ------------------------------------
 integer,intent(in) :: iqptfull,nband
 type(elph_type),intent(inout) :: elph_ds
 real(dp),intent(in) :: FSintweight(nband,elph_ds%nFSkpt)
 real(dp),intent(in) :: FSkpt(3,elph_ds%nFSkpt)
 real(dp),intent(in) :: displ_red(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp),intent(in) :: eigvec(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp),intent(in) :: h1_mat_el(2,nband,nband,elph_ds%nbranch,elph_ds%nFSkpt)
 real(dp),intent(in) :: phfrq_tmp(elph_ds%nbranch),spqpt(3,elph_ds%nqpt)
 real(dp),intent(in) :: wf(elph_ds%nbranch)
 
!Local variables-------------------------------
 integer :: eivec,goodkpq,i1,i2,iFSkpt,iFSkptq,ib1,ib2,ibranch,ier,ii,jbranch
 integer :: kbranch
 real(dp) :: qphnrm,res,sd1,sd2,ss
 character(len=4) :: dirstr,qptstr
 character(len=fnlen) :: xsffilnam
 real(dp) :: accum_mat(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: accum_mat2(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: gam_now2(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: gkk_qpt_tmp(2,elph_ds%nbranch,elph_ds%nbranch,elph_ds%nFSkpt)
 real(dp) :: gkq(2),kpt(3),redkpt(3),tmp_mat(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: tmp_mat2(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp),allocatable :: matrx(:,:),val(:),vec(:,:,:),zhpev1(:,:),zhpev2(:)
 
! *************************************************************************

#ifdef __VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif

  gkk_qpt_tmp(:,:,:,:) = zero
  accum_mat(:,:,:) = zero
  accum_mat2(:,:,:) = zero

  if (elph_ds%doscalprod == 1) then
     do iFSkpt=1,elph_ds%nFSkpt
        !
        ! Find equivalent point for k+q which are stored in hdr1%kptns
        !    (supposes the qgrid is a subgrid of the kgrid)
        goodkpq = 0
        do iFSkptq=1,elph_ds%nFSkpt
           kpt(:) = FSkpt(:,iFSkpt)+spqpt(:,iqptfull)-FSkpt(:,iFSkptq)
           call canon9(kpt(1),redkpt(1),res)
           call canon9(kpt(2),redkpt(2),res)
           call canon9(kpt(3),redkpt(3),res)

           ss=redkpt(1)**2+redkpt(2)**2+redkpt(3)**2

           if (ss < tol6) then
!DEBUG
!write (*,*) 'Found index = ', iFSkptq, ' for k+q = ', kpt
!ENDDEBUG
              goodkpq = 1
              exit
           else
              cycle
           end if
        end do
        if (goodkpq == 0) then
           write (*,*) 'read_gkk : Error: k+q vector is not in grid of FS kpoints'
           write (*,*) 'The qpoint grid is probably not a sub-grid of the kpoint one.'
           stop
        end if


        tmp_mat(:,:,:) = zero

        do ib1=1,elph_ds%nFSband
           !  weights for distance from the fermi surface
           sd1 = FSintweight(ib1,iFSkpt)

           do ib2=1,elph_ds%nFSband
              !  weights for distance from the fermi surface
              sd2 = FSintweight(ib2,iFSkptq)

              ! phonon mode index for displ_red
              do jbranch=1,elph_ds%nbranch

                 gkq(:) = zero
                 ! Do scalar product of displacement vector by reduced perturbations
                 ! loops over atoms and red directions
                 do ibranch=1,elph_ds%nbranch
                    ! should this be displ_red complex conjugated? Yes from DECAFT
                    gkq(1) = gkq(1)&
                         & + displ_red(1,ibranch,jbranch)*h1_mat_el(1,ib2,ib1,ibranch,iFSkpt) &
                         & + displ_red(2,ibranch,jbranch)*h1_mat_el(2,ib2,ib1,ibranch,iFSkpt)
                    gkq(2) = gkq(2)&
                         & + displ_red(1,ibranch,jbranch)*h1_mat_el(2,ib2,ib1,ibranch,iFSkpt) &
                         & - displ_red(2,ibranch,jbranch)*h1_mat_el(1,ib2,ib1,ibranch,iFSkpt)
                 end do

                 !  Add weights for integration over FS and wf = (1/sqrt(2 omega))
                 tmp_mat(1,jbranch,jbranch) = &
                      &   tmp_mat(1,jbranch,jbranch) + &
                      &   sd1 * sd2 * wf(jbranch)**2 * (gkq(1)**2+gkq(2)**2)
              end do

!DEBUG
!write (*,'(a,a,4i8,2E16.6)') ' normsq_gkq : ',&
!     &  'ib1,ib2,iFSkpt,iqptfull,tmp_mat(1,1,1) = ',&
!     &   ib1,ib2,iFSkpt,iqptfull,tmp_mat(:,1,1)
!write (*,*) '###', elph_ds%gkk_qpt(:,hdr1%pertcase,ib1,ib2,iFSkpt)
!ENDDEBUG
           end do
        end do
        ! END loop over bands

!DEBUG
!if (abs(spqpt(1,iqptfull))+abs(spqpt(2,iqptfull)-half)+abs(spqpt(3,iqptfull)-half) < tol6) then
!write (*,'(a,9(2E18.6,x))') 'normsq_gkq : g2fq as in decaft = ', &
!     &  (tmp_mat(1,jbranch,jbranch),jbranch=1,elph_ds%nbranch)
!end if
!ENDDEBUG

        accum_mat(:,:,:) = accum_mat(:,:,:) + tmp_mat(:,:,:)

        ! 28 Apr 2004  PI factor added to wf in read_gkk
        ! 15 jun 2004  put back into wf
        ! Add factor of 2 pi omega(branch) to gamma. Still in diagonal normal mode basis
        do jbranch=1,elph_ds%nbranch
          tmp_mat(:,jbranch,jbranch) = two * pi * phfrq_tmp(jbranch) * tmp_mat(:,jbranch,jbranch)
        end do

        accum_mat2(:,:,:) = accum_mat2(:,:,:) + tmp_mat(:,:,:)

        ! summed over bands, now turn to cartesian coordinates

        !  Final Gamma matrix (hermitian) = E * D_g * E^{+}
        !    Where E^{+} is the hermitian conjugate of the eigenvector matrix E
        !    And D_g is the diagonal matrix of values of gamma for this qpoint

        ! Here tmp_mat is indexed with real phonon modes (not atom+idir)
        ! turn tmp_mat to atom+cartesian coordinates (instead of normal coordinates for qpoint)
        tmp_mat2(:,:,:) = zero
        do ibranch =1,elph_ds%nbranch
           do jbranch =1,elph_ds%nbranch
              !do kbranch=1,elph_ds%nbranch
              !   tmp_mat2(1,ibranch,jbranch) = eigvec(1,ibranch,kbranch) * &
              !        &  tmp_mat(1,kbranch,jbranch)
              !   tmp_mat2(2,ibranch,jbranch) = eigvec(2,ibranch,kbranch) * &
              !        &  tmp_mat(1,kbranch,jbranch)
              !end do
              tmp_mat2(1,ibranch,jbranch) = tmp_mat2(1,ibranch,jbranch) + &
                   &  eigvec(1,ibranch,jbranch) * &
                   &  tmp_mat(1,jbranch,jbranch)
              tmp_mat2(2,ibranch,jbranch) = tmp_mat2(2,ibranch,jbranch) + &
                   &  eigvec(2,ibranch,jbranch) * &
                   &  tmp_mat(1,jbranch,jbranch)
           end do
        end do
!DEBUG
!write (*,'(a,9(2E16.6,x))') 'normsq_gkq : tmp_mat2 = ', tmp_mat2
!ENDDEBUG
        tmp_mat(:,:,:) = zero
        do ibranch =1,elph_ds%nbranch
           do jbranch =1,elph_ds%nbranch
              do kbranch=1,elph_ds%nbranch
                 tmp_mat(1,ibranch,jbranch) =  tmp_mat(1,ibranch,jbranch) &
                      &  +tmp_mat2(1,ibranch,kbranch) * &
                      &        eigvec(1,jbranch,kbranch)&
                      &  +tmp_mat2(2,ibranch,kbranch) * &
                      &        eigvec(2,jbranch,kbranch)
                 ! Again, here eigvec is transposed and complexconjugated.
                 tmp_mat(2,ibranch,jbranch) = tmp_mat(2,ibranch,jbranch) &
                      &  -tmp_mat2(1,ibranch,kbranch) * &
                      &        eigvec(2,jbranch,kbranch)  &
                      &  +tmp_mat2(2,ibranch,kbranch) * &
                      &        eigvec(1,jbranch,kbranch)
              end do
           end do
        end do


!DEBUG
!if (abs(spqpt(1,iqptfull))+abs(spqpt(2,iqptfull)-half)+abs(spqpt(3,iqptfull)-half) < tol6) then
!write (*,'(a)') 'normsq_gkq : tmp_mat after turning as gmatq from decaft = '
!do ibranch =1,elph_ds%nbranch
!   write (*,'(3(2E18.6,x))') (tmp_mat(:,jbranch,ibranch),jbranch=1,elph_ds%nbranch)
!end do
!
!   ! Diagonalize gamma matrix at qpoint (complex matrix). Copied from phfrq3
!   ier=0
!   ii=1
!   allocate(matrx(2,(elph_ds%nbranch*(elph_ds%nbranch+1))/2))
!   do i2=1,elph_ds%nbranch
!      do i1=1,i2
!         matrx(1,ii)=tmp_mat(1,i1,i2)
!         matrx(2,ii)=tmp_mat(2,i1,i2)
!         ii=ii+1
!      end do
!   end do
!   allocate(zhpev1(2,2*elph_ds%nbranch-1),zhpev2(3*elph_ds%nbranch-2))
!   allocate(val(elph_ds%nbranch),vec(2,elph_ds%nbranch,elph_ds%nbranch))
!#if defined T3E
!   call CHPEV ('V','U',elph_ds%nbranch,matrx,val,vec,elph_ds%nbranch,zhpev1,&
!        &    zhpev2,ier)
!#else
!   call ZHPEV ('V','U',elph_ds%nbranch,matrx,val,vec,elph_ds%nbranch,zhpev1,&
!        &    zhpev2,ier)
!#endif
!
!   write (*,*) ' normsq_gkq : eigenvalues = '
!   write (*,'(3E18.6)') val
!
!   deallocate(matrx,zhpev1,zhpev2,vec,val)
!end if
!ENDDEBUG

        !! gamma matrix contribution in cartesian coordinates (ie interpolatable form)
        ! gamma matrix contribution in reduced coordinates (ie interpolatable form)
        gkk_qpt_tmp(:,:,:,iFSkpt) = gkk_qpt_tmp(:,:,:,iFSkpt) + tmp_mat(:,:,:)

        !accum_mat(:,:,:) = accum_mat(:,:,:) + tmp_mat(:,:,:)
     end do
     !  END loop over FSkpt

  else
     ! else elph_ds%doscalprod == 0  Interpolate on the pure "matrix of matrix elements"
     ! and do the scalar products later.

     do iFSkpt=1,elph_ds%nFSkpt
        !
        ! Find equivalent point for k+q which are stored in hdr1%kptns
        !    (supposes the qgrid is a subgrid of the kgrid)
        goodkpq = 0
        do iFSkptq=1,elph_ds%nFSkpt
           kpt(:) = FSkpt(:,iFSkpt)+spqpt(:,iqptfull)-FSkpt(:,iFSkptq)
           call canon9(kpt(1),redkpt(1),res)
           call canon9(kpt(2),redkpt(2),res)
           call canon9(kpt(3),redkpt(3),res)

           ss=redkpt(1)**2+redkpt(2)**2+redkpt(3)**2

           if (ss < tol6) then
!DEBUG
!write (*,*) 'Found index = ', iFSkptq, ' for k+q = ', kpt
!ENDDEBUG
              goodkpq = 1
              exit
           else
              cycle
           end if
        end do
        if (goodkpq == 0) then
           write (*,*) 'read_gkk : Error: k+q vector is not in grid of FS kpoints'
           write (*,*) 'The qpoint grid is probably not a sub-grid of the kpoint one.'
           stop
        end if


        tmp_mat(:,:,:) = zero



        !  tmp_mat = \sum_{ib1,ib2} \langle k+q \mid H^{(1)}_{q,\tau_i,\alpha_i} \mid k   \rangle
        !                     \cdot \langle k   \mid H^{(1)}_{q,\tau_j,\alpha_j} \mid k+q \rangle
        !   where ibranch -> \tau_i,\alpha_i  and  jbranch -> \tau_j,\alpha_j

        do ib1=1,elph_ds%nFSband
           !  weights for distance from the fermi surface
           sd1 = FSintweight(ib1,iFSkpt)

           do ib2=1,elph_ds%nFSband
              !  weights for distance from the fermi surface
              sd2 = FSintweight(ib2,iFSkptq)

              ! phonon mode index for displ_red
              do jbranch=1,elph_ds%nbranch

                 !
                 !08/04/2004 do scalar product later in interpolate routine.
                 !    Use complex conj of h1_mat_el(jbranch)
                 !  WARNING: wf is counted here and not in mode-dependent case when scalar product with
                 !     displ is carried out. Is a constant, so dont care too much right now
                 do ibranch=1,elph_ds%nbranch
                    tmp_mat(1,ibranch,jbranch) = tmp_mat(1,ibranch,jbranch) + &
                         &   sd1*sd2*wf(jbranch)*&
                         &   ( h1_mat_el(1,ib2,ib1,ibranch,iFSkpt) * &
                         &     h1_mat_el(1,ib2,ib1,jbranch,iFSkpt) + &
                         &     h1_mat_el(2,ib2,ib1,ibranch,iFSkpt) * &
                         &     h1_mat_el(2,ib2,ib1,jbranch,iFSkpt) )
                    tmp_mat(2,ibranch,jbranch) = tmp_mat(2,ibranch,jbranch) + &
                         &   sd1*sd2*wf(jbranch)*&
                         &   ( h1_mat_el(2,ib2,ib1,ibranch,iFSkpt) * &
                         &     h1_mat_el(1,ib2,ib1,jbranch,iFSkpt) - &
                         &     h1_mat_el(1,ib2,ib1,ibranch,iFSkpt) * &
                         &     h1_mat_el(2,ib2,ib1,jbranch,iFSkpt) )
                 end do
              end do
           end do
        end do
        ! END loops over bands


        !! gamma matrix contribution in cartesian coordinates (ie interpolatable form)
        ! gamma matrix contribution in reduced coordinates (ie interpolatable form)
        gkk_qpt_tmp(:,:,:,iFSkpt) = gkk_qpt_tmp(:,:,:,iFSkpt) + tmp_mat(:,:,:)

        accum_mat(:,:,:) = accum_mat(:,:,:) + tmp_mat(:,:,:)
     end do
     !  END loop over FSkpt

  end if
  ! end if flag for doing scalar product now.

  accum_mat(:,:,:) = accum_mat(:,:,:)/elph_ds%nFSkpt
  accum_mat2(:,:,:) = accum_mat2(:,:,:)/elph_ds%nFSkpt

  ! immediately calculate linewidths:
  write (*,*) 'summed accum_mat = '
  write (*,'(3(2E18.6,x))')accum_mat
  write (*,*) 'summed accum_mat2 = '
  write (*,'(3(2E18.6,x))')  (accum_mat2(:,ii,ii),ii=1,elph_ds%nbranch)
  write (*,*) 'displ_red  = '
  write (*,'(3(2E18.6,x))') displ_red

  !!DEBUG
  if (elph_ds%doscalprod == 1) then

     ! Diagonalize gamma matrix at qpoint (complex matrix). Copied from phfrq3
     ier=0
     ii=1
     allocate(matrx(2,(elph_ds%nbranch*(elph_ds%nbranch+1))/2))
     do i2=1,elph_ds%nbranch
        do i1=1,i2
           matrx(1,ii)=accum_mat2(1,i1,i2)
           matrx(2,ii)=accum_mat2(2,i1,i2)
           ii=ii+1
        end do
     end do
     allocate(zhpev1(2,2*elph_ds%nbranch-1),zhpev2(3*elph_ds%nbranch-2))
     allocate(val(elph_ds%nbranch),vec(2,elph_ds%nbranch,elph_ds%nbranch))
#if defined T3E
     call CHPEV ('V','U',elph_ds%nbranch,matrx,val,vec,elph_ds%nbranch,zhpev1,&
          &    zhpev2,ier)
#else
     call ZHPEV ('V','U',elph_ds%nbranch,matrx,val,vec,elph_ds%nbranch,zhpev1,&
          &    zhpev2,ier)
#endif

     write (*,*) ' normsq_gkq : accumulated eigenvalues = '
     write (*,'(3E18.6)') val

     deallocate(matrx,zhpev1,zhpev2,vec,val)

  else if (elph_ds%doscalprod == 0) then
     gam_now2(:,:,:) = zero

     do jbranch=1,elph_ds%nbranch
        do ibranch=1,elph_ds%nbranch
           do kbranch=1,elph_ds%nbranch
              ! gam = displ gam_red displ^{*T}
              gam_now2(1,jbranch,jbranch) = gam_now2(1,jbranch,jbranch) + &
                &  displ_red(1,ibranch,jbranch)*accum_mat(1,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                & +displ_red(1,ibranch,jbranch)*accum_mat(2,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                & +displ_red(2,ibranch,jbranch)*accum_mat(1,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                & -displ_red(2,ibranch,jbranch)*accum_mat(2,ibranch,kbranch)*displ_red(1,kbranch,jbranch)
              gam_now2(2,jbranch,jbranch) = gam_now2(2,jbranch,jbranch)   &
                & +displ_red(2,ibranch,jbranch)*accum_mat(2,ibranch,kbranch)*displ_red(2,kbranch,jbranch) &
                & +displ_red(2,ibranch,jbranch)*accum_mat(1,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                & +displ_red(1,ibranch,jbranch)*accum_mat(2,ibranch,kbranch)*displ_red(1,kbranch,jbranch) &
                & -displ_red(1,ibranch,jbranch)*accum_mat(1,ibranch,kbranch)*displ_red(2,kbranch,jbranch)
           end do
        end do
     end do

     write (*,'(3(2E14.6,x))') (gam_now2(:,jbranch,jbranch), jbranch=1,elph_ds%nbranch)

  end if
  !!ENDDEBUG

  ! save gkk_qpt_tmp, eventually to disk
  if (elph_ds%gkqwrite == 0) then
     elph_ds%gkk_qpt(:,:,:,:,iqptfull) = gkk_qpt_tmp(:,:,:,:)
  else
     !rewind (elph_ds%unitgkq)
     !do iqpt=1,iqptfull-1
     !   read (elph_ds%unitgkq) gkk_qpt_tmp2
     !end do
     write (*,'(a,i4,2(3E16.6,3x))') 'gkk_qpt_tmp ', &
          &     iqptfull, gkk_qpt_tmp(:,:,1,1:10)
     write (elph_ds%unitgkq,REC=iqptfull) gkk_qpt_tmp
  end if


end subroutine normsq_gkq
!!***
